/*
    ES Knot
    Copyright 2011 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#include <QtOpenGL>
#include "qtmain.h"
#include "knot2.h"

void debug(const char* str) {
	printf("%s\n", str);
}

KnotWindow::KnotWindow() {
	k = NULL;
	distance = 0;
	pressed = false;
	timer.start();
}

void KnotWindow::initializeGL() {
#ifndef Q_WS_S60
	printf("vendor: %s\n", glGetString(GL_VENDOR));
	printf("renderer: %s\n", glGetString(GL_RENDERER));
	printf("version: %s\n", glGetString(GL_VERSION));
	printf("extensions: %s\n", glGetString(GL_EXTENSIONS));
#endif
}

void KnotWindow::resizeGL(int w, int h) {
	if (k) {
		k->setSize(w, h);
		k->reinit();
	} else {
		k = new Knot(w, h);
		k->setTexMode(Knot::TEXSPEC);
		k->setUpdateVertices(true);
		k->enableRotate(true);
	}
}

void KnotWindow::paintGL() {
	k->draw(timer.elapsed());
}

void KnotWindow::mousePressEvent(QMouseEvent* event) {
	k->enableRotate(false);
	prevPos = event->pos();
	distance = 0;
}

void KnotWindow::mouseReleaseEvent(QMouseEvent* event) {
	Q_UNUSED(event);
	if (distance < 50)
		k->startMorph();
	k->enableRotate(true);
}

void KnotWindow::mouseMoveEvent(QMouseEvent* event) {
	if (event->buttons() & Qt::LeftButton) {
		QPoint diff = event->pos() - prevPos;
		k->rotate(diff.x(), diff.y());
		distance += diff.manhattanLength();
	}
	prevPos = event->pos();
}

int main(int argc, char* argv[]) {
	QApplication app(argc, argv);
	KnotWindow* window = new KnotWindow();
	QTimer timer;
	timer.setInterval(30);
	QObject::connect(&timer, SIGNAL(timeout()), window, SLOT(update()));
	timer.start();
#ifdef Q_WS_S60
	window->showMaximized();
#else
	window->show();
#endif
	return app.exec();
}

