package st.martin.esknot;

import java.lang.reflect.Method;
import java.util.List;

import android.app.Activity;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.Window;

public class KnotActivity extends Activity implements SensorEventListener {
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);

		mGLSurfaceView = new GLSurfaceView(this);
		mRenderer = new KnotRenderer();

		try {
			Class<?> factoryInterfaceClass = Class.forName("android.opengl.GLSurfaceView$EGLContextFactory");
			Class<?> chooserInterfaceClass = Class.forName("android.opengl.GLSurfaceView$EGLConfigChooser");
			Class<?> factoryClass = Class.forName("st.martin.esknot.GLES2EGLContextFactory");
			Class<?> surfaceViewClass = mGLSurfaceView.getClass();
			Method method = surfaceViewClass.getMethod("setEGLContextFactory", factoryInterfaceClass);
			Object factory = factoryClass.newInstance();
			method.invoke(mGLSurfaceView, factory);
			Method method2 = surfaceViewClass.getMethod("setEGLConfigChooser", chooserInterfaceClass);
			method2.invoke(mGLSurfaceView, factory);
		} catch (Throwable t) {
			t.printStackTrace();
		}

		mGLSurfaceView.setRenderer(mRenderer);
//		mGLSurfaceView.setRenderMode(GLSurfaceView.RENDERMODE_WHEN_DIRTY);
		setContentView(mGLSurfaceView);
	}
	
	private void initSensors() {
		SensorManager manager = ((SensorManager)getSystemService(SENSOR_SERVICE)); 
		List<Sensor> sensors = manager.getSensorList(Sensor.TYPE_ACCELEROMETER);
		if (sensors.size() > 0) {
			manager.registerListener(this, sensors.get(0), SensorManager.SENSOR_DELAY_GAME, null);
			mRenderer.setHasGravity(true);
		} else {
			mRenderer.setHasGravity(false);
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
		mGLSurfaceView.onResume();
		initSensors();
	}

	@Override
	protected void onPause() {
		super.onPause();
		mGLSurfaceView.onPause();
		SensorManager manager = ((SensorManager)getSystemService(SENSOR_SERVICE)); 
		manager.unregisterListener(this);
	}

	public boolean onTouchEvent(MotionEvent event) {
		if (mRenderer != null) {
			if (mRenderer.onTouchEvent(event)) {
				mGLSurfaceView.requestRender();
				return true;
			}
		}
		return super.onTouchEvent(event);
	}

	public boolean onCreateOptionsMenu(Menu menu) {
		boolean result = super.onCreateOptionsMenu(menu);
		menu.add(Menu.NONE, MENU_TEXTURE_MODE, Menu.NONE, "Texture mode");
		menu.add(Menu.NONE, MENU_VERTEX_UPDATING, Menu.NONE, "Vertex updating");
		menu.add(Menu.NONE, MENU_RADIAL_BLUR, Menu.NONE, "Radial blur");
		menu.add(Menu.NONE, MENU_MOTION_BLUR, Menu.NONE, "Motion blur");
		return result;
	}

	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case MENU_TEXTURE_MODE:
			mRenderer.toggleTextureMode();
			return true;
		case MENU_VERTEX_UPDATING:
			mRenderer.toggleVertexUpdating();
			return true;
		case MENU_RADIAL_BLUR:
			mRenderer.toggleRadialBlur();
			return true;
		case MENU_MOTION_BLUR:
			mRenderer.toggleMotionBlur();
			return true;
		default:
			break;
		}
		return super.onOptionsItemSelected(item);
	}

	private GLSurfaceView mGLSurfaceView;
	private KnotRenderer mRenderer;
	
	private static final int MENU_TEXTURE_MODE = 0;
	private static final int MENU_VERTEX_UPDATING = 1;
	private static final int MENU_RADIAL_BLUR = 2;
	private static final int MENU_MOTION_BLUR = 3;

	public void onAccuracyChanged(Sensor arg0, int arg1) {
	}

	public void onSensorChanged(SensorEvent event) {
		float[] values = event.values;
		Sensor sensor = event.sensor;
		if (sensor.getType() != Sensor.TYPE_ACCELEROMETER)
			return;
		if (values.length < 3)
			return;
		mRenderer.setGravityVector(values[0], values[1], values[2]);
	}
}
